package com.dd.admin.business.api;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.file.FileNameUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dd.admin.business.card.domain.CardDto;
import com.dd.admin.business.card.domain.CardVo;
import com.dd.admin.business.note.domain.NoteDto;
import com.dd.admin.business.note.domain.NoteVo;
import com.dd.admin.business.note.entity.Note;
import com.dd.admin.business.note.service.NoteService;
import com.dd.admin.business.noteImg.domain.NoteImgDto;
import com.dd.admin.business.noteImg.domain.NoteImgVo;
import com.dd.admin.business.noteImg.entity.NoteImg;
import com.dd.admin.business.noteImg.service.NoteImgService;
import com.dd.admin.business.upNotes.domain.UpNotesDto;
import com.dd.admin.business.upNotes.domain.UpNotesVo;
import com.dd.admin.business.upNotes.service.UpNotesService;
import com.dd.admin.common.aop.operationLog.aop.OperLog;
import com.dd.admin.common.aop.operationLog.aop.OperType;
import com.dd.admin.common.model.result.ResultBean;
import com.dd.admin.common.security.SecurityUtil;
import com.dd.admin.common.security.jwt.JwtTokenUtil;
import com.dd.admin.common.utils.AddressUtils;
import com.dd.admin.common.utils.IPUtils;
import com.dd.admin.common.utils.RandomXiaohongshuAuthorName;
import com.dd.admin.common.utils.StringUtil;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@RestController
public class ApiController {
    @Autowired
    NoteService noteService;
    @Autowired
    HttpServletRequest request;
    @Autowired
    NoteImgService noteImgService;
    @Autowired
    UpNotesService upNotesService;
    @Autowired
    private JwtTokenUtil jwtTokenUtil;
    @ApiOperation(value = "获取所有笔记")
    @ApiOperationSupport(order = 1)
    @GetMapping("/api/notes")
    @OperLog(operModule = "获取所有笔记", operType = OperType.QUERY, operDesc = "获取所有笔记")
    public ResultBean<IPage<NoteVo>> page(NoteDto noteDto) {
        // 1. 通过noteService根据传入的NoteDto参数查询笔记的分页信息，获取包含笔记数据的分页对象
        IPage<NoteVo> pageInfo = noteService.selectNotePage(noteDto);

        // 2. 创建一个用于存储当前用户点赞笔记信息的列表，初始化为空
        List<UpNotesVo> upNotesVos = new ArrayList<>();

        // 3. 从请求头中获取名为"token"的token字符串，用于后续解析获取用户ID
        String token = request.getHeader("token");
        String authorId = "";
        try {
            // 4. 尝试使用jwtTokenUtil工具类从获取到的token中解析出用户名（这里假设用户名就是用户ID），如果解析成功则赋值给authorId变量
            authorId = jwtTokenUtil.getUsernameFromToken(token);
        } catch (Exception e) {
            // 5. 如果在解析token过程中出现异常，此处捕获异常但不做任何处理，authorId仍为空字符串，后续可根据此情况决定是否继续相关业务逻辑
        }

        // 6. 判断解析出的用户ID是否不为空（即是否成功获取到了用户ID）
        if (StringUtil.isNotEmpty(authorId)) {
            // 7. 如果获取到了用户ID，创建一个UpNotesDto对象，用于设置查询条件
            UpNotesDto upNotesDto = new UpNotesDto();
            // 8. 将获取到的用户ID设置到UpNotesDto对象中，作为查询当前用户点赞笔记的条件（表示要查询该用户点赞的笔记）
            upNotesDto.setFollowId(authorId);
            // 9. 通过upNotesService根据设置好的查询条件，查询当前用户点赞的笔记列表，并将结果赋值给upNotesVos列表
            upNotesVos = upNotesService.selectUpNotesList(upNotesDto);
        }

        // 10. 通过upNotesService查询所有笔记的点赞数量统计信息，获取一个包含各笔记点赞数量相关信息的列表
        List<UpNotesVo> selectAllUpCount = upNotesService.selectAllUpCount();

        // 11. 获取前面查询到的笔记分页信息中的具体笔记记录列表，后续要对这些笔记记录进行相关属性的设置操作
        List<NoteVo> records = pageInfo.getRecords();
        // 12. 为了能在下面的lambda表达式中使用upNotesVos变量，创建一个final修饰的变量来引用它，因为lambda表达式中访问的外部变量需要是final或者事实上的final（不可重新赋值）
        List<UpNotesVo> finalUpNotesVos = upNotesVos;
        // 13. 同样，为了在lambda表达式中使用authorId变量，创建一个final修饰的变量来引用它
        String finalAuthorId = authorId;

        // 14. 使用Java 8的Stream API对每条笔记记录进行遍历处理
        records.stream().forEach(noteVo -> {
            // 15. 判断当前用户ID是否不为空（即是否获取到了用户ID且有效），如果为空则不进行当前用户对该笔记点赞状态的判断设置操作
            if (StringUtil.isNotEmpty(finalAuthorId)) {
                // 16. 在当前用户点赞笔记列表（finalUpNotesVos）中，通过Stream的filter操作筛选出与当前遍历的笔记（noteVo）的ID相等的点赞笔记记录，然后获取第一个匹配的记录（如果有的话），否则返回null
                UpNotesVo upNote = finalUpNotesVos.stream().filter(upNotesVo -> noteVo.getNoteId().equals(upNotesVo.getNoteId())).findFirst().orElse(null);
                // 17. 如果找到了对应的点赞笔记记录（即upNote不为null），说明当前用户点赞了这条笔记，将笔记的IsUp属性设置为true，表示已点赞状态
                if (upNote!= null) {
                    noteVo.setIsUp(Boolean.TRUE);
                }
            }

            // 18. 在所有笔记的点赞数量统计信息列表（selectAllUpCount）中，通过Stream的filter操作筛选出与当前遍历的笔记（noteVo）的ID相等的记录，然后获取第一个匹配的记录（如果有的话），否则返回null
            UpNotesVo noteCount = selectAllUpCount.stream().filter(upNotesVo -> upNotesVo.getNoteId().equals(noteVo.getNoteId())).findFirst().orElse(null);
            // 19. 如果找到了对应的点赞数量统计记录（即noteCount不为null），将笔记的UpCount属性设置为该记录中的点赞数量，从而为每条笔记设置对应的点赞数量信息
            if (noteCount!= null) {
                noteVo.setUpCount(noteCount.getUpCount());
            }
        });

        // 20. 将处理好的包含笔记信息（已设置点赞状态和点赞数量）的分页对象包装在ResultBean中返回，以便前端或其他调用方获取处理后的结果数据
        return ResultBean.success(pageInfo);
    }

    @ApiOperation(value = "获取ip地址")
    @ApiOperationSupport(order = 1)
    @GetMapping("/api/getIpAddress")
    @OperLog(operModule = "获取ip地址",operType = OperType.QUERY,operDesc = "获取单个笔记")
    public ResultBean<String> getIpAddress() {
        String ipAddr = IPUtils.getIpAddr(request);
        String realAddress = AddressUtils.getRealAddress(ipAddr);
        return ResultBean.success(realAddress);
    }

    @ApiOperation(value = "获取单个笔记")
    @ApiOperationSupport(order = 1)
    @GetMapping("/api/getNote")
    @OperLog(operModule = "获取单个笔记",operType = OperType.QUERY,operDesc = "获取单个笔记")
    public ResultBean<NoteVo> getNoteById(NoteDto noteDto) {
        Note note = noteService.getById(noteDto.getNoteId());
        NoteVo noteVo = BeanUtil.copyProperties(note, NoteVo.class);
        noteVo.setCreateTimeStr(noteVo.getCreateTime());

        NoteImgDto noteImgDto = new NoteImgDto();
        noteImgDto.setNoteId(note.getNoteId());
        List<NoteImgVo> noteImgVos = noteImgService.selectNoteImgList(noteImgDto);
        List<String> imageList = noteImgVos.stream().map(NoteImgVo::getImgUrl).collect(Collectors.toList());
        noteVo.setImgList(imageList);
        return ResultBean.success(noteVo);
    }





}
